local function downscale(picture)
    if not picture then return end

    if type(picture) == "table" and picture.layers then
        for _, layer in pairs(picture.layers or {}) do
            downscale(layer)
        end
        return
    end

    picture.scale = (picture.scale or 1) / 2
end

local function vector_downscale(vector)
    if not vector then return end

    if vector.x and type(vector.x) == "number" then vector.x = vector.x / 2 end
    if vector.y and type(vector.y) == "number" then vector.y = vector.y / 2 end
    if vector[1] and type(vector[1]) == "number" then vector[1] = vector[1] / 2 end
    if vector[2] and type(vector[2]) == "number" then vector[2] = vector[2] / 2 end

    for _, subvector in pairs(vector) do
        if type(subvector) == "table" then
            vector_downscale(subvector)
        end
    end
end

local roboport = table.deepcopy(data.raw["roboport"]["roboport"])
roboport.name = "factory-construction-roboport"
roboport.collision_box = {{-0.4, -0.4}, {0.4, 0.4}}
roboport.selection_box = {{-0.5, -0.5}, {0.5, 0.5}}
roboport.recharging_light.size = roboport.recharging_light.size / 2
roboport.charging_station_count_affected_by_quality = true
roboport.logistics_radius = 4
roboport.radar_range = 0
downscale(roboport.base)
downscale(roboport.base_patch)
downscale(roboport.frozen_patch)
downscale(roboport.base_animation)
downscale(roboport.door_animation_up)
downscale(roboport.door_animation_down)
downscale(roboport.recharging_animation)
vector_downscale(roboport.charging_station_shift)
vector_downscale(roboport.stationing_offset)
vector_downscale(roboport.charging_offsets)

local storage_chest = table.deepcopy(data.raw["logistic-container"]["passive-provider-chest"])
storage_chest.name = "factory-construction-chest"

local requester_chest = table.deepcopy(data.raw.container["steel-chest"])
requester_chest.name = "factory-requester-chest"
requester_chest.collision_box = {{0, 0}, {0, 0}}
requester_chest.selection_box = nil

for _, prototype in pairs{roboport, storage_chest, requester_chest} do
    prototype.collision_mask = {layers = {}}
    prototype.minable = nil
    flags = {"placeable-neutral", "player-creation"}
end

data:extend{roboport, storage_chest, requester_chest}